/* ===================================================================================================
* WARNING – This file is part of the base implementation for WebMaker, so it should not be edited or changed for any project. 
* These files are replaced if a project is re-imported to the WebMaker Studio or migrated to a new version of the product. 
* For guidance on 'How do I override or clone Hyfinity webapp files such as CSS & javascript?', please read the following relevant FAQ entry: 
* http://www.hyfinity.net/faq/index/solution_id/1113
==================================================================================================== */
/**
 * Treeview functions - Handles expanding and collapsing of treeview sections. Controlled by toggling the class names.
 *
 * @param container {Group_Name} (mandatory) 
 * @param animate   {boolean} (optional) If 'true' the expanding/collapsing will appear gradually. If 'false' it will be performed immediately. 
 * @private 
 * @author Hyfinity Limited
 */
function treeItemClicked(container, animate)
{
    var entryId = container.getAttribute('id');

    if ((animate == null) || (typeof(animate) == 'undefined') || animate)
        hyf.util.toggleComponent(entryId + "_content", null, true);
    else
        hyf.util.toggleComponent(entryId + "_content", null, false);

    if (dojo.hasClass(container, 'node-collapsible'))
    {
        dojo.removeClass(container, 'node-collapsible');
        dojo.addClass(container, 'node-expandable');
    }
    else
    {
        dojo.removeClass(container, 'node-expandable');
        dojo.addClass(container, 'node-collapsible');
    }
}

/**
 * Expands the specified tree enry
 * @param container The SPAN element representing the tree entry to expand
 * @private 
 * @author Hyfinity Limited
 */
function expandTreeEntry(container)
{
    var entryId = container.getAttribute('id');

    if (dojo.hasClass(container, 'node-expandable'))
    {
        hyf.util.toggleComponent(entryId + "_content", 'show', true);
        dojo.removeClass(container, 'node-expandable');
        dojo.addClass(container, 'node-collapsible');
    }
}

/**
 * Expands all parent entries in the tree so that the provided entry is visible
 * The provided entry itself will not be expanded.
 * @param elem The SPAN element representing the tree entry to be visible
 * @private
 * @author Hyfinity Limited
 */
function expandTree(elem)
{
    var parents = new Array();
    var tempElem = elem;

    while (!dojo.hasClass(tempElem, 'treeviewTop'))
    {
        if (tempElem.tagName.toUpperCase() == 'UL')
        {
            parents[parents.length] = tempElem;
        }
        tempElem = tempElem.parentNode;
    }

    for(var i = parents.length; i > 0; --i)
    {
        var list = parents[i-1];
        var sp = getPreviousElementSibling(list);
        expandTreeEntry(sp);
    }
}


/**
 * Set the selected item within the tree. Designed to cater for multiple trees with positional tags used to determine 'local' value to be set
 * @param e Event, typically onclick
 * @param nodeThe anchor that was clicked initially, then walk up the parent tree
 * @param leaf_value Value of the selected leaf
 * @return false to ensure the anchor href is not executed
 * @private
 * @author Hyfinity Limited
 */
function setSelectedLeaf(e, node, leaf_value)
{
    var parent = node.parentNode;
    if(dojo.hasClass(parent, "treeviewTop"))
    {
        // top-level UL with class treeviewTop has been found.
        // The immediate container should be the repeat div tree_data_context and its first sibling should be the hidden data field container
        var selectedItem = parent.parentNode.nextSibling.getElementsByTagName("input")[0];
        selectedItem.value = leaf_value;
        if (typeof(selectedItem.onchange) == "function")
        {
            selectedItem.onchange();
        }
    }
    else if (parent != null)
    {
        setSelectedLeaf(e, parent, leaf_value);
    }
    else
    {
        //end of parent walk
    }
    return false;
}
